//////////////////////////////////////////////////////////////////////////
// Find the real roots of
//       ax^2 + bx + c = 0
// using,
//      Method 1:  x1 = (-b + sqrt(b^2 - 4ac))/2a
//                 x2 = (-b - sqrt(b^2 - 4ac))/2a
// and, from Numerical Recipes,
//      Method 2:  q = -0.5*(b + sgn(b)*sqrt(b^2 - 4ac))
//                 x1 = q/a;  x2 = c/q.
//
//                  thomas l. gibson  October 16, 2001
//
// Updated by:      thomas l. gibson  January 5, 2007
//
//////////////////////////////////////////////////////////////////////////
#include <iostream>
#include <iomanip>
#include <cmath>
#include <cstdlib>
#include <cassert>
using namespace std;
int quadrat_1(long double a, long double b, long double c, long double rad);
int quadrat_2(long double a, long double b, long double c, long double rad, int sgnb);
int main() 
{
  long double a,b,c,q,test,rad;
  int sgnb,check;
  bool doit;
  char ask;
  doit = true;
  while(doit) 
  {
    system("clear");
    cout << "Quadratic Equation Solver: \n";
    cout << "\t Input a,b,c for ax^2 + bx + c = 0 \n";
    cin >> a >> b >> c;
    test = b*b - 4.0*a*c;
    sgnb = (int) copysign(1.0,b);    //Gnu Math routine---thanks Pat
    if(a == 0.0) 
    {
      cout << "Sorry, but we're not dividing by zero today. \n";
      cout << "\t a = 0 not allowed. \n";
    }
    else if(test < 0.0) 
    {
      cout << "Sorry, we're only doing real roots today. \n";
      cout << "\t b^2 < 4*a*c not allowed. \n";
    }
    rad = sqrt(test);
    check = quadrat_1(a,b,c,rad);
    if (check != 0)
    {
      cout << "Error! function quadrat_1 returned nonzero value of "
           << check << endl;
    }
    check = quadrat_2(a,b,c,rad,sgnb);
    if (check != 0)
    {
      cout << "Error! function quadrat_2 returned nonzero value of "
           << check << endl;
    }
    cout << "\n\n";
    cout << "Solve another one? (y/n) ";
    cin >> ask;
    if((ask != 'y') && (ask != 'Y')) 
    {
      doit = false;
    }
  }
  return 0;
}
//////////////////////////////////////////////////////////////////////////
// Quadratic Equation Solver: Method 1 (Traditional Form)
//
//                thomas l. gibson  October 16, 2001
//////////////////////////////////////////////////////////////////////////
int quadrat_1(long double a, long double b, long double c, long double rad) 
{
  long double root_1,root_2;
  root_1 = -0.5*(rad + b)/a;
  root_2 = 0.5*(rad - b)/a;
  cout << "Method 1 (Traditional Form): \n"
       << setprecision(16)
       << "\t root 1 = " << root_1 << "\n"
       << "\t root 2 = " << root_2 << "\n";
  return 0;
}
//////////////////////////////////////////////////////////////////////////
// Quadratic Equation Solver: Method 2 (Numerical Recipes)
//
//               thomas l. gibson  October 16, 2001
//////////////////////////////////////////////////////////////////////////
int quadrat_2(long double a, long double b, long double c, long double rad, int sgnb) {  
  long double root_1,root_2,q;
  q = -0.5*(b + sgnb*rad);
  root_1 = q/a;
  assert(q != 0.0);       // Make sure that q is not zero
  root_2 = c/q;
  cout << "Method 2 (Numerical Recipes): \n"
       << setprecision(16)
       << "\t root 1 = " << root_1 << "\n"
       << "\t root 2 = " << root_2 << "\n";
  return 0;
}
//////////////////////////////////////////////////////////////////////////
